#include <fstream>
#include <iostream>

#include <boost/iostreams/filtering_streambuf.hpp>
#include <boost/iostreams/filter/gzip.hpp>
#include <boost/iostreams/device/file.hpp>
namespace io = boost::iostreams;



namespace {
  const char *  infile = "msg.gz";
  const char *  outfile = "msg.part";
}



class logged_gzip_decompressor: public io::gzip_decompressor
{
public:
  logged_gzip_decompressor() {
    std::cout << "gzip_decompressor has been constructed at: " << static_cast<void*>(this) << std::endl;
  }
  logged_gzip_decompressor(const logged_gzip_decompressor& other)
    : io::gzip_decompressor(other)
  {
    std::cout << "gzip_decompressor has been copy constructed from: " << static_cast<const void*>(&other)
              << " at: " << static_cast<void*>(this) << std::endl;
  }
  ~logged_gzip_decompressor() {
    std::cout << "gzip_decompressor has been destroyed at: " << static_cast<void*>(this) << std::endl;
  }
  logged_gzip_decompressor& operator= (const logged_gzip_decompressor& other) {
    if (&other == this)
      return *this;
    io::gzip_decompressor::operator= (other);
    std::cout << "gzip_decompressor at: " << static_cast<void*>(this)
              << " has been assigned to: " << static_cast<const void*>(&other) << std::endl;
    return *this;
  }
  
};



size_t copy_some(std::streambuf& in, std::streambuf& out)
{
  enum {BUFSIZE = 50};
  char buf[BUFSIZE];
  std::streamsize result = in.sgetn(buf, BUFSIZE);
  result = out.sputn(buf, result);
  return result;
}



int main() 
{
  try {
    std::filebuf in;
    in.open(infile, std::ios_base::in | std::ios_base::binary);
    
    io::filtering_streambuf<io::output> out;
    out.push(logged_gzip_decompressor());
    out.push(io::file_sink(outfile));
    
    copy_some(in, out);
  }
  catch (std::exception& err) {
    std::cout << "std::exception has been caught: " << err.what() << std::endl;
  }
  catch (...) {
    std::cout << "unknown exception has been caught" << std::endl;
  }
  return 0;
}
